﻿using FreeScheduler;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using NPOI.SS.Formula.Functions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using ZhonTai.Admin.Core.Consts;
using ZhonTai.Admin.Core.Dto;
using ZhonTai.Admin.Domain.Org;
using ZhonTai.Admin.Domain.Project;
using ZhonTai.Admin.Domain.Tenant;
using ZhonTai.Admin.Services.Project.Dto;
using ZhonTai.Common.Extensions;
using ZhonTai.DynamicApi;
using ZhonTai.DynamicApi.Attributes;

namespace ZhonTai.Admin.Services.Project
{
    /// <summary>
    /// 项目价格设置服务
    /// </summary>
    [Order(10)]
    [DynamicApi(Area = AdminConsts.AreaName)]
    public partial class ProjectPriceService : BaseService, IProjectPriceService, IDynamicApi
    {
        private readonly IProjectRepository _projectRepository;
        private readonly IProjectPriceRepository _projectPriceRepository;
        private readonly IProjectConfigRepository _projectConfigRepository;
        private readonly IOrgRepository _orgRepository;
        private readonly ITenantRepository _tenantRepository;

        public ProjectPriceService(IProjectRepository projectRepository, IProjectPriceRepository projectPriceRepository, IProjectConfigRepository projectConfigRepository, IOrgRepository orgRepository, ITenantRepository tenantRepository)
        {
            _projectRepository = projectRepository;
            _projectPriceRepository = projectPriceRepository;
            _projectConfigRepository = projectConfigRepository;
            _orgRepository = orgRepository;
            _tenantRepository = tenantRepository;
        }
        #region 公司默认抽成比例
        /// <summary>
        /// 查询公司抽成比例
        /// </summary>        
        /// <returns></returns>
        [HttpGet]
        public async Task<CompanyDrawOutput> GetCompanyDrawAsync()
        {
            await ProjectPriceInitAsync();

            var list = await _projectConfigRepository.Select
                .DisableGlobalFilter(FilterNames.Tenant)
                .Where(m => m.ProjectId == 0 && m.ProjectPriceId == 0 && m.DrawType == 1 && m.Status == 1)
                .OrderByDescending(m => m.EffectDate)
            .ToListAsync(a => new
            {
                a.Id,
                a.DrawRatio,
                a.EffectDate
            });
            CompanyDrawOutput output = new CompanyDrawOutput();
            if (list.Count > 1)
            {
                DateTime dtnow = DateTime.Today;
                var next = list[0];
                var current = list[1];
                output.CurrentRatio = current.DrawRatio.Value;
                output.NextRatio = next.DrawRatio;
                output.NextEffectDate = next.EffectDate;
            }
            else
            {
                output.CurrentRatio = list[0].DrawRatio.Value;
            }
            return output;
        }
        /// <summary>
        /// 修改公司抽成比例
        /// </summary>        
        /// <returns></returns>
        [HttpPost]
        public async Task EditCompanyDrawAsync(CompanyDrawInput input)
        {
            CheckDrawRatio(input.DrawRatio);

            CheckDrawRatioEffect(input.EffectDate);

            var dtDay = DateTime.Today;
            //删除旧的未生效记录            
            var listDeleteId = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.DrawType == 1 && m.ProjectId == 0 && m.ProjectPriceId == 0 && m.Status == 1 && m.EffectDate > dtDay)
            .ToListAsync(m => m.Id);
            if (listDeleteId.Count > 0)
            {
                await _projectConfigRepository.SoftDeleteAsync(m => listDeleteId.Contains(m.Id));
            }
            //添加新的比例
            await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
            {
                ProjectId = 0,
                ProjectPriceId = 0,
                DrawRatio = input.DrawRatio,
                EffectDate = input.EffectDate,
                Status = 1,
                DrawType = 1,
                DrawWay = 1,
                TenantId = 0
            });
        }
        #endregion

        #region 公司设置项目抽成比例
        /// <summary>
        /// 查询项目分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<GetProjectPricePageOutput>> GetProjectPageAsync(PageInput<GetProjectPricePageInput> input)
        {
            var list = await _projectRepository.Select.DisableGlobalFilter(FilterNames.Tenant)
            .Count(out var total)
            .OrderByDescending(true, a => a.Id)
            .Page(input.CurrentPage, input.PageSize)
            .ToListAsync(a => new GetProjectPricePageOutput()
            {
                Id = a.Id,
                Name = a.Name,
                Logo = a.Logo,
                Status = a.Status,
                Prices = _projectPriceRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.ProjectId == a.Id).ToList<PriceGetPageOutput_Price>(
                    b=>new PriceGetPageOutput_Price() { 
                        Id=b.Id,
                        Name=b.Name,
                        Price=b.Price,
                        DrawPriceWay=b.DrawPriceWay,
                        DrawRatio=b.DrawRatio,
                        DrawPrice=b.DrawPrice,
                        IsExistNextInfo = _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m=>m.DrawType==2&&m.ProjectPriceId==b.Id&&m.Status==1).Count()
                    })
            });

            var data = new PageOutput<GetProjectPricePageOutput>()
            {
                List = Mapper.Map<List<GetProjectPricePageOutput>>(list),
                Total = total
            };

            return data;
        }
        /// <summary>
        /// 项目价格设价
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<long> SetProjectPriceAsync(ProjectPriceSetInput input)
        {
            //验证项目价格信息是否存在
            var price = await _projectPriceRepository.Select.DisableGlobalFilter(FilterNames.Tenant).From<ProjectEntity>().LeftJoin((pp, p) => pp.ProjectId == p.Id)
                .Where((pp, p) => pp.Id == input.Id).ToOneAsync((pp, p) => new
                {
                    pp.Id,
                    pp.ProjectId,
                    pp.Price,
                    p.Status
                });
            if (!(price?.Id > 0))
            {
                throw ResultOutput.Exception("信息不存在,请刷新后重试！");
            }
            if (!new int[] { 1, 2 }.Contains(input.DrawPriceWay))
            {
                throw ResultOutput.Exception("无效的设价方式！");
            }
            decimal? amount = input.DrawPrice;
            if (input.DrawPriceWay == 1)
            {
                CheckDrawRatio(input.DrawRatio);

                amount = GetDrawAmount(price.Price, input.DrawRatio.Value);
            }
            else if (input.DrawPriceWay == 2)
            {
                //抽成金额
                if (input.DrawPrice > price.Price || input.DrawPrice < 0)
                {
                    throw ResultOutput.Exception("设置有效的抽成金额");
                }
            }
            if (price.Status != 1)
            {
                CheckDrawRatioEffect(input.EffectDate);
            }

            //项目没有上架可以直接启用设价信息
            if (price.Status == 1)
            {
                await _projectPriceRepository.UpdateDiy.SetDto(new
                {
                    DrawPriceWay = input.DrawPriceWay,
                    DrawRatio = input.DrawRatio,
                    DrawPrice = amount,
                }).Where(m => m.Id == price.Id).DisableGlobalFilter(FilterNames.Tenant).ExecuteAffrowsAsync();

                //新增记录
                await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
                {
                    ProjectId = price.ProjectId,
                    ProjectPriceId = price.Id,
                    Status = 1,
                    DrawWay = input.DrawPriceWay,
                    DrawRatio = input.DrawRatio,
                    DrawAmount = amount,
                    DrawType = 2,
                    TenantId = 0,
                    IsDeleted = true
                });
            }
            else
            {
                //覆盖已经存在未生效的
                var listDeleteId = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.ProjectId == price.ProjectId && m.ProjectPriceId == price.Id && m.Status == 1 && m.EffectDate > DateTime.Today && m.DrawType == 2)
               .ToListAsync(m => m.Id);
                if (listDeleteId.Count > 0)
                {
                    await _projectConfigRepository.SoftDeleteAsync(m => listDeleteId.Contains(m.Id));
                }

                //新增
                var effectDate = input.EffectDate.HasValue ? input.EffectDate : DateTime.Today.AddDays(1);
                await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
                {
                    ProjectId = price.ProjectId,
                    ProjectPriceId = price.Id,
                    Status = 1,
                    DrawWay = input.DrawPriceWay,
                    DrawRatio = input.DrawRatio,
                    DrawAmount = amount,
                    EffectDate = effectDate,
                    DrawType = 2,
                    TenantId = 0
                });
            }
            return 1;
        }
        #endregion

        #region 公司设置平台抽成比例
        /// <summary>
        /// 查询平台单独设价分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<TenantPriceGetPageOutput>> GetTenantDrawPageAsync(PageInput<TenantPriceGetPageInput> input)
        {
            var projectId = input?.Filter?.ProjectId;
            if (!projectId.HasValue)
            {
                projectId = 0;
            }
            //项目价格信息
            var projectprice = _projectPriceRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.ProjectId == projectId).ToList(m => new
            {
                m.Id,
                m.Name,
                m.Price
            });
            //平台信息
            var list = await _orgRepository.Select.DisableGlobalFilter(FilterNames.Tenant)
            .Count(out var total)
            .OrderByDescending(true, a => a.Id)
            .Page(input.CurrentPage, input.PageSize)
            .ToListAsync(a => new TenantPriceGetPageOutput()
            {
                Id = a.TenantId.Value,
                Name = a.Name,
                prices = _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.ProjectId == projectId && m.TenantId == a.TenantId && m.Status == 1 && m.DrawType == 3).ToList(m => new TenantPriceGetPageOutput_Price
                {
                    ProjectId = m.ProjectId,
                    ProjectPriceId = m.ProjectPriceId,
                    DrawPriceWay = m.DrawWay,
                    DrawRatio = m.DrawRatio,
                    DrawPrice = m.DrawAmount,
                    EffectDate=m.EffectDate
                })
            });
            //项目信息同平台信息合并
            var dtToday = DateTime.Today;
            foreach (var item in list)
            {
                List<TenantPriceGetPageOutput_Price> prices = new List<TenantPriceGetPageOutput_Price>();
                foreach (var it in projectprice)
                {
                    var listItem=item.prices.Where(m => m.ProjectPriceId == it.Id).ToList();
                    var tprice = listItem.Where(m=>m.EffectDate<= dtToday).FirstOrDefault();
                    if (tprice != null)
                    {
                        prices.Add(new TenantPriceGetPageOutput_Price()
                        {
                            ProjectId = tprice.ProjectId,
                            ProjectPriceId = tprice.ProjectPriceId,
                            Name = it.Name,
                            Price = it.Price,
                            DrawPriceWay = tprice.DrawPriceWay,
                            DrawRatio = tprice.DrawRatio,
                            DrawPrice = tprice.DrawPrice,
                            IsExistNextInfo= listItem.Count>1?1:0
                        });
                    }
                    else
                    {
                        prices.Add(new TenantPriceGetPageOutput_Price()
                        {
                            ProjectId = projectId.Value,
                            ProjectPriceId = it.Id,
                            Name = it.Name,
                            Price = it.Price,
                            DrawPriceWay = 0,
                            IsExistNextInfo=listItem.Count>0?1:0
                        });
                    }
                }
                item.prices = prices;
            }

            var data = new PageOutput<TenantPriceGetPageOutput>()
            {
                List = Mapper.Map<List<TenantPriceGetPageOutput>>(list),
                Total = total
            };

            return data;
        }
        /// <summary>
        /// 平台项目设置价格
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<long> SetTenantDrawAsync(TenantDrawSetInput input)
        {
            if (!new[] { 1, 2 }.Contains(input.DrawPriceWay))
            {
                throw ResultOutput.Exception("无效的设价方式！");
            }
            //验证平台是否存在
            var tenant = await _tenantRepository.Select.DisableGlobalFilter(FilterNames.Tenant)
                .Where(m => m.Id == input.TenatntId).ToOneAsync();
            if (!(tenant?.Id > 0))
            {
                throw ResultOutput.Exception("平台不存在,请刷新后重试！");
            }

            //验证项目价格信息是否存在
            var price = await _projectPriceRepository.Select.DisableGlobalFilter(FilterNames.Tenant)
                .From<ProjectEntity>().LeftJoin((pp, p) => pp.ProjectId == p.Id)
                .Where((pp, p) => pp.Id == input.ProjectPriceId).ToOneAsync((pp, p) => new
                {
                    pp.Id,
                    pp.Price,
                    pp.ProjectId,
                    p.Status
                });
            if (!(price?.Id > 0))
            {
                throw ResultOutput.Exception("项目信息不存在,请刷新后重试！");
            }

            var amount = input.DrawPrice;
            if (input.DrawPriceWay == 1)
            {
                //抽成比例
                CheckDrawRatio(input.DrawRatio);

                amount = GetDrawAmount(price.Price, input.DrawRatio.Value);

            }
            else if (input.DrawPriceWay == 2)
            {
                //抽成金额
                if (input.DrawPrice > price.Price || input.DrawPrice < 0)
                {
                    throw ResultOutput.Exception("设置有效的抽成金额");
                }

            }
            //生效时间验证
            if (price.Status != 1)
            {
                CheckDrawRatioEffect(input.EffectDate);
            }
            //待上线项目直接生效
            if (price.Status == 1)
            {
                await _projectConfigRepository.UpdateDiy.DisableGlobalFilter(FilterNames.Tenant)
                    .SetDto(new
                    {
                        IsDeleted = true
                    })
                    .Where(m => m.ProjectId == price.ProjectId && m.ProjectPriceId == price.Id && m.TenantId == input.TenatntId && m.Status == 1 && m.DrawType == 3)
                    .ExecuteAffrowsAsync();
                //新增
                var effectDate = DateTime.Today;
                await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
                {
                    ProjectId = price.ProjectId,
                    ProjectPriceId = price.Id,
                    Status = 1,
                    DrawWay = input.DrawPriceWay,
                    DrawRatio = input.DrawRatio,
                    DrawAmount = amount,
                    EffectDate = effectDate,
                    DrawType = 3,
                    TenantId = input.TenatntId
                });
            }
            else
            {
                //覆盖已经存在未生效的
                var listDeleteId = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant)
                    .Where(m => m.ProjectId == price.ProjectId && m.ProjectPriceId == price.Id && m.Status == 1 && m.TenantId == input.TenatntId && m.EffectDate > DateTime.Today && m.DrawType == 3)
               .ToListAsync(m => m.Id);
                if (listDeleteId.Count > 0)
                {
                    await _projectConfigRepository.SoftDeleteAsync(m => listDeleteId.Contains(m.Id));
                }

                //新增
                var effectDate = input.EffectDate;
                await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
                {
                    ProjectId = price.ProjectId,
                    ProjectPriceId = price.Id,
                    Status = 1,
                    DrawWay = input.DrawPriceWay,
                    DrawRatio = input.DrawRatio,
                    DrawAmount = amount,
                    EffectDate = effectDate,
                    DrawType = 3,
                    TenantId = input.TenatntId
                });
            }
            return 1;
        }
        /// <summary>
        /// 下次生效信息
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<DrawInfoOutput> GetDrawInfoAsync(DrawInfoInput input)
        {
            if (!new[] { 2, 3 }.Contains(input.DrawType))
            {
                throw ResultOutput.Exception("无效的抽成类型！");
            }
            DrawInfoOutput result   =new DrawInfoOutput();
            //项目抽成信息
            if (input.DrawType == 2)
            {
                result = await _projectConfigRepository.Select
                    .DisableGlobalFilter(FilterNames.Tenant)
                    .Where(m => m.DrawType == 2 && m.ProjectPriceId == input.ProjectPriceId && m.EffectDate > DateTime.Today)
                    .OrderBy(m => m.EffectDate)
                    .ToOneAsync(m=>new DrawInfoOutput{ 
                        DrawWay= m.DrawWay,
                        DrawRatio= m.DrawRatio,
                        DrawAmount= m.DrawAmount,
                        EffectDate= m.EffectDate
                    });                
            }
            else
            {
                result = await _projectConfigRepository.Select
                   .DisableGlobalFilter(FilterNames.Tenant)
                   .Where(m => m.DrawType == 3 && m.ProjectPriceId == input.ProjectPriceId &&m.TenantId==input.TenantId && m.EffectDate > DateTime.Today)
                   .OrderBy(m => m.EffectDate)
                   .ToOneAsync(m => new DrawInfoOutput
                   {
                       DrawWay = m.DrawWay,
                       DrawRatio = m.DrawRatio,
                       DrawAmount = m.DrawAmount,
                       EffectDate = m.EffectDate
                   });                
            }
            return result;
        }
        #endregion

        #region 定时任务 每天凌晨执行一次
        /// <summary>
        /// 执行公司抽成任务
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [AllowAnonymous]
        public async Task ExecuteTaskCompanyDraw()
        {
            var dtDay = DateTime.Today;
            var list = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.DrawType == 1 && m.Status == 1 && m.EffectDate == dtDay).ToListAsync();
            if (list != null && list.Count == 1)
            {
                //之前的数据失效
                var result = await _projectConfigRepository.UpdateDiy
                    .DisableGlobalFilter(FilterNames.Tenant)
                    .SetDto(new { Status = 2 })
                    .Where(m => m.ProjectId == 0 && m.ProjectPriceId == 0 && m.TenantId == 0 && m.DrawType == 1 && m.Status == 1 && m.EffectDate < dtDay)
                    .ExecuteAffrowsAsync();
                if (result > 0)
                {
                    throw ResultOutput.Exception($"成功执行{result}条数据");
                }
                else
                {
                    throw ResultOutput.Exception("执行失败");
                }
            }
            else
            {
                throw ResultOutput.Exception("无需执行数据");
            }
        }
        /// <summary>
        /// 执行抽成项目任务
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [AllowAnonymous]
        public async Task ExecuteTaskProjectDraw()
        {
            var dtToday = DateTime.Today;
            var list = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.DrawType == 2 && m.Status == 1 && m.EffectDate ==dtToday ).ToListAsync();
            if (list != null)
            {
                //数据挪项目中
                foreach (var item in list)
                {
                    await _projectPriceRepository.UpdateDiy.SetDto(new
                    {
                        DrawPriceWay=item.DrawWay,
                        DrawRatio=item.DrawRatio,
                        DrawPrice=item.DrawAmount
                    }).Where(m => m.Id == item.ProjectPriceId && m.ProjectId == item.ProjectId)
                    .DisableGlobalFilter(FilterNames.Tenant)
                    .ExecuteAffrowsAsync();
                }

                //之前的数据失效
                var result = await _projectConfigRepository.UpdateDiy
                    .DisableGlobalFilter(FilterNames.Tenant)
                    .SetDto(new { Status = 2 })
                    .Where(m => m.DrawType == 2 && m.Status == 1 && m.EffectDate < dtToday)
                    .ExecuteAffrowsAsync();
                if (result > 0)
                {
                    throw ResultOutput.Exception($"成功执行{result}条数据");
                }
                else
                {
                    throw ResultOutput.Exception("执行失败");
                }
            }
            else
            {
                throw ResultOutput.Exception("无需执行数据");
            }
        }

        /// <summary>
        /// 执行抽成平台任务
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [AllowAnonymous]
        public async Task ExecuteTasDrawTenant()
        {
            var dtToday = DateTime.Today;
            var list = await _projectConfigRepository.Select.DisableGlobalFilter(FilterNames.Tenant).Where(m => m.DrawType == 3 && m.Status == 1 && m.EffectDate == dtToday).ToListAsync();
            if (list != null)
            {
                //之前的数据失效
                var result = await _projectConfigRepository.UpdateDiy
                    .DisableGlobalFilter(FilterNames.Tenant)
                    .SetDto(new { Status = 2 })
                    .Where(m => m.DrawType == 3 && m.Status == 1 && m.EffectDate < dtToday)
                    .ExecuteAffrowsAsync();
                if (result > 0)
                {
                    throw ResultOutput.Exception($"成功执行{result}条数据");
                }
                else
                {
                    throw ResultOutput.Exception("执行失败");
                }
            }
            else
            {
                throw ResultOutput.Exception("无需执行数据");
            }
        }
        #endregion


        #region 校验
        /// <summary>
        /// 校验抽成比例
        /// </summary>
        /// <param name="drawRatio"></param>
        private void CheckDrawRatio(decimal? drawRatio)
        {
            if (!drawRatio.HasValue)
            {
                throw ResultOutput.Exception("抽成比例有效范围在0-100");
            }
            //抽成比例
            if (drawRatio < 0 || drawRatio > 100)
            {
                throw ResultOutput.Exception("抽成比例有效范围在0-100");
            }
        }
        /// <summary>
        /// 校验抽成比例生效时间
        /// </summary>
        /// <param name="effectDate"></param>
        private void CheckDrawRatioEffect(DateTime? effectDate)
        {
            if (!effectDate.HasValue)
            {
                throw ResultOutput.Exception("请设置生效时间");
            }
            //生效时间
            if (effectDate <= DateTime.Today)
            {
                throw ResultOutput.Exception("生效日期，最短为次日生效");
            }
        }
        /// <summary>
        /// 抽成金额
        /// </summary>
        public static decimal GetDrawAmount(decimal price, decimal ratio)
        {
            var amount = UtilConvertExtension.ToDecimalCutWithN(price * ratio * 0.01m, 0, 1);
            return amount;
        }
        /// <summary>
        /// 初始化公司抽成信息
        /// </summary>        
        /// <returns></returns>
        private async Task ProjectPriceInitAsync()
        {
            var count = await _projectConfigRepository.Select.Where(m => m.ProjectId == 0 && m.ProjectPriceId == 0).CountAsync();
            if (count <= 0)
            {
                await _projectConfigRepository.InsertAsync(new ProjectConfigEntity()
                {
                    ProjectId = 0,
                    ProjectPriceId = 0,
                    DrawRatio = 5,
                    EffectDate = DateTime.Today,
                    TenantId = -1,
                    Status = 1,
                    DrawWay = 1
                });
            }
        }
        #endregion
    }
}